@@ -34,6 +34,7 @@ pub enum OperationType {
     InitializeRebase,
     MakeGraph,
     ProcessEvents,
+    PushBranches,
     QueryWorkingCopy,
     ReadingFromCache,
     RebaseCommits,
@@ -66,6 +67,7 @@ impl ToString for OperationType {
             OperationType::GetUpstreamPatchIds => "Enumerating patch IDs",
             OperationType::InitializeRebase => "Initializing rebase",
             OperationType::MakeGraph => "Examining local history",
+            OperationType::PushBranches => "Pushing branches",
             OperationType::ProcessEvents => "Processing events",
             OperationType::QueryWorkingCopy => "Querying the working copy",
             OperationType::ReadingFromCache => "Reading from cache",
