@@ -1,21 +1,21 @@
-use std::borrow::Cow;
 use std::collections::{HashMap, HashSet};
 
-use std::ffi::{OsStr, OsString};
 use std::fmt::Write;
 use std::path::PathBuf;
 use std::time::SystemTime;
 
+use bstr::BString;
 use eyre::Context;
-use os_str_bytes::OsStrBytes;
 use tracing::warn;
 
-use crate::core::check_out::{check_out_commit, CheckOutCommitOptions};
+use crate::core::check_out::{check_out_commit, CheckOutCommitOptions, CheckoutTarget};
 use crate::core::effects::Effects;
 use crate::core::eventlog::{EventLogDb, EventTransactionId};
 use crate::core::formatting::{printable_styled_string, Pluralize};
 use crate::core::repo_ext::RepoExt;
-use crate::git::{GitRunInfo, MaybeZeroOid, NonZeroOid, Repo, ResolvedReferenceInfo};
+use crate::git::{
+    GitRunInfo, MaybeZeroOid, NonZeroOid, ReferenceName, Repo, ResolvedReferenceInfo,
+};
 use crate::util::ExitCode;
 
 use super::plan::RebasePlan;
@@ -37,7 +37,7 @@ pub fn move_branches<'a>(
     // first error, but we don't know which references we successfully committed
     // in that case. Instead, we just do things non-atomically and record which
     // ones succeeded. See https://github.com/libgit2/libgit2/issues/5918
-    let mut branch_moves: Vec<(NonZeroOid, MaybeZeroOid, &OsStr)> = Vec::new();
+    let mut branch_moves: Vec<(NonZeroOid, MaybeZeroOid, &ReferenceName)> = Vec::new();
     let mut branch_move_err: Option<eyre::Error> = None;
     'outer: for (old_oid, names) in branch_oid_to_names.iter() {
         let new_oid = match rewritten_oids_map.get(old_oid) {
@@ -62,14 +62,17 @@ pub fn move_branches<'a>(
                     }
                 };
 
-                for name in names {
-                    if let Err(err) =
-                        repo.create_reference(name, new_commit.get_oid(), true, "move branches")
-                    {
-                        branch_move_err = Some(err);
+                for reference_name in names {
+                    if let Err(err) = repo.create_reference(
+                        reference_name,
+                        new_commit.get_oid(),
+                        true,
+                        "move branches",
+                    ) {
+                        branch_move_err = Some(eyre::eyre!(err));
                         break 'outer;
                     }
-                    branch_moves.push((*old_oid, MaybeZeroOid::NonZero(*new_oid), name));
+                    branch_moves.push((*old_oid, MaybeZeroOid::NonZero(*new_oid), reference_name));
                 }
             }
 
@@ -78,7 +81,7 @@ pub fn move_branches<'a>(
                     match repo.find_reference(name) {
                         Ok(Some(mut reference)) => {
                             if let Err(err) = reference.delete() {
-                                branch_move_err = Some(err);
+                                branch_move_err = Some(eyre::eyre!(err));
                                 break 'outer;
                             }
                         }
@@ -86,7 +89,7 @@ pub fn move_branches<'a>(
                             warn!(?name, "Reference not found, not deleting")
                         }
                         Err(err) => {
-                            branch_move_err = Some(err);
+                            branch_move_err = Some(eyre::eyre!(err));
                             break 'outer;
                         }
                     };
@@ -96,22 +99,13 @@ pub fn move_branches<'a>(
         }
     }
 
-    let branch_moves_stdin: Vec<u8> = branch_moves
+    let branch_moves_stdin: String = branch_moves
         .into_iter()
-        .flat_map(|(old_oid, new_oid, name)| {
-            let mut line = Vec::new();
-            line.extend(old_oid.to_string().as_bytes());
-            line.push(b' ');
-            line.extend(new_oid.to_string().as_bytes());
-            line.push(b' ');
-            line.extend(name.to_raw_bytes().iter());
-            line.push(b'\n');
-            line
+        .map(|(old_oid, new_oid, name)| {
+            format!("{old_oid} {new_oid} {name}\n", name = name.as_str())
         })
         .collect();
-    let branch_moves_stdin =
-        OsStrBytes::from_raw_bytes(branch_moves_stdin).wrap_err("Encoding branch moves stdin")?;
-    let branch_moves_stdin = OsString::from(branch_moves_stdin);
+    let branch_moves_stdin = BString::from(branch_moves_stdin);
     git_run_info.run_hook(
         effects,
         repo,
@@ -165,7 +159,7 @@ pub fn check_out_updated_head(
             // can happen, but if so, just use that branch.
             ResolvedReferenceInfo {
                 oid: None,
-                reference_name: Some(Cow::Borrowed(reference_name)),
+                reference_name: Some(reference_name.clone()),
             }
         }
 
@@ -218,7 +212,7 @@ pub fn check_out_updated_head(
                     let oid = repo.resolve_reference(&reference)?.oid;
                     ResolvedReferenceInfo {
                         oid,
-                        reference_name: Some(Cow::Borrowed(reference_name)),
+                        reference_name: Some(reference_name.clone()),
                     }
                 }
 
@@ -239,7 +233,7 @@ pub fn check_out_updated_head(
         return Ok(ExitCode(0));
     }
 
-    let checkout_target: Cow<OsStr> = match &checkout_target {
+    let checkout_target: CheckoutTarget = match &checkout_target {
         ResolvedReferenceInfo {
             oid: None,
             reference_name: None,
@@ -248,7 +242,7 @@ pub fn check_out_updated_head(
         ResolvedReferenceInfo {
             oid: Some(oid),
             reference_name: None,
-        } => Cow::Owned(OsString::from(oid.to_string())),
+        } => CheckoutTarget::Oid(*oid),
 
         ResolvedReferenceInfo {
             oid: _,
@@ -256,10 +250,11 @@ pub fn check_out_updated_head(
         } => {
             // FIXME: we could check to see if the OIDs are the same and, if so,
             // reattach or detach `HEAD` manually without having to call `git checkout`.
-            match checkout_target.get_branch_name()? {
-                Some(branch_name) => Cow::Owned(branch_name),
-                None => Cow::Borrowed(reference_name),
-            }
+            let checkout_target = match checkout_target.get_branch_name()? {
+                Some(branch_name) => branch_name,
+                None => reference_name.as_str(),
+            };
+            CheckoutTarget::Reference(ReferenceName::from(checkout_target))
         }
     };
 
@@ -269,7 +264,7 @@ pub fn check_out_updated_head(
         repo,
         event_log_db,
         event_tx_id,
-        Some(&checkout_target),
+        Some(checkout_target),
         check_out_commit_options,
     )?;
     Ok(result)
@@ -345,9 +340,9 @@ impl MergeConflictInfo {
 
 mod in_memory {
     use std::collections::HashMap;
-    use std::ffi::OsString;
     use std::fmt::Write;
 
+    use bstr::{BString, ByteSlice};
     use eyre::Context;
     use tracing::{instrument, warn};
 
@@ -395,10 +390,18 @@ mod in_memory {
                 .iter()
                 .find_map(|command| match command {
                     RebaseCommand::Merge {
+                        replacement_commit_oid: None,
                         commit_oid,
                         commits_to_merge: _,
                     } => Some(commit_oid),
-                    RebaseCommand::CreateLabel { .. }
+                    RebaseCommand::Merge {
+                        // If we know exactly what tree to use, we don't
+                        // actually have to carry out a merge in memory.
+                        replacement_commit_oid: Some(_),
+                        commit_oid: _,
+                        commits_to_merge: _,
+                    }
+                    | RebaseCommand::CreateLabel { .. }
                     | RebaseCommand::Reset { .. }
                     | RebaseCommand::Pick { .. }
                     | RebaseCommand::RegisterExtraPostRewriteHook
@@ -516,7 +519,7 @@ mod in_memory {
                         &CherryPickFastOptions {
                             reuse_parent_tree_if_possible: true,
                         },
-                    )? {
+                    ) {
                         Ok(rebased_commit) => rebased_commit,
                         Err(CherryPickFastError::MergeConflict { conflicting_paths }) => {
                             return Ok(RebaseInMemoryResult::MergeConflict(MergeConflictInfo {
@@ -524,10 +527,11 @@ mod in_memory {
                                 conflicting_paths,
                             }))
                         }
+                        Err(other) => eyre::bail!(other),
                     };
 
                     let commit_message = commit_to_apply.get_message_raw()?;
-                    let commit_message = commit_message.to_str().ok_or_else(|| {
+                    let commit_message = commit_message.to_str().with_context(|| {
                         eyre::eyre!(
                             "Could not decode commit message for commit: {:?}",
                             commit_to_apply_oid
@@ -590,18 +594,105 @@ mod in_memory {
                 }
 
                 RebaseCommand::Merge {
+                    replacement_commit_oid: None,
                     commit_oid,
                     commits_to_merge: _,
                 } => {
                     warn!(
                         ?commit_oid,
-                        "BUG: Merge commit should have been detected when starting in-memory rebase"
+                        "BUG: Merge commit without replacement should have been detected when starting in-memory rebase"
                     );
                     return Ok(RebaseInMemoryResult::CannotRebaseMergeCommit {
                         commit_oid: *commit_oid,
                     });
                 }
 
+                RebaseCommand::Merge {
+                    replacement_commit_oid: Some(replacement_commit_oid),
+                    commit_oid,
+                    commits_to_merge,
+                } => {
+                    let current_commit = repo
+                        .find_commit_or_fail(current_oid)
+                        .wrap_err("Finding current commit")?;
+                    let commit_to_apply = repo
+                        .find_commit_or_fail(*replacement_commit_oid)
+                        .wrap_err("Finding commit to apply")?;
+                    i += 1;
+
+                    let commit_description = printable_styled_string(
+                        effects.get_glyphs(),
+                        commit_to_apply.friendly_describe(effects.get_glyphs())?,
+                    )?;
+                    let commit_num = format!("[{}/{}]", i, num_picks);
+                    progress.notify_progress(i, num_picks);
+
+                    progress
+                        .notify_status(format!("Applying merge commit: {}", commit_description));
+                    let commit = repo.find_commit_or_fail(*replacement_commit_oid)?;
+                    let commit_tree = commit.get_tree()?;
+                    let commit_message = commit_to_apply.get_message_raw()?;
+                    let commit_message = commit_message.to_str().with_context(|| {
+                        eyre::eyre!(
+                            "Could not decode commit message for commit: {:?}",
+                            replacement_commit_oid,
+                        )
+                    })?;
+
+                    progress
+                        .notify_status(format!("Committing to repository: {}", commit_description));
+                    let committer_signature = if *preserve_timestamps {
+                        commit_to_apply.get_committer()
+                    } else {
+                        commit_to_apply.get_committer().update_timestamp(*now)?
+                    };
+                    let parents = {
+                        let mut result = vec![current_commit];
+                        for parent in commits_to_merge {
+                            let parent = match parent {
+                                OidOrLabel::Oid(oid) => repo.find_commit_or_fail(*oid)?,
+                                OidOrLabel::Label(label) => {
+                                    let oid = labels.get(label).ok_or_else(|| {
+                                        eyre::eyre!(
+                                            "Label {label} could not be resolved to a commit"
+                                        )
+                                    })?;
+                                    repo.find_commit_or_fail(*oid)?
+                                }
+                            };
+                            result.push(parent);
+                        }
+                        result
+                    };
+                    let rebased_commit_oid = repo
+                        .create_commit(
+                            None,
+                            &commit_to_apply.get_author(),
+                            &committer_signature,
+                            commit_message,
+                            &commit_tree,
+                            parents.iter().collect(),
+                        )
+                        .wrap_err("Applying rebased commit")?;
+
+                    let commit_description = printable_styled_string(
+                        effects.get_glyphs(),
+                        repo.friendly_describe_commit_from_oid(
+                            effects.get_glyphs(),
+                            rebased_commit_oid,
+                        )?,
+                    )?;
+                    rewritten_oids.push((*commit_oid, MaybeZeroOid::NonZero(rebased_commit_oid)));
+                    current_oid = rebased_commit_oid;
+
+                    writeln!(
+                        effects.get_output_stream(),
+                        "{} Committed as: {}",
+                        commit_num,
+                        commit_description
+                    )?;
+                }
+
                 RebaseCommand::SkipUpstreamAppliedCommit { commit_oid } => {
                     i += 1;
                     let commit_num = format!("[{}/{}]", i, num_picks);
@@ -726,7 +817,7 @@ mod in_memory {
             .iter()
             .map(|(old_oid, new_oid)| format!("{} {}\n", old_oid, new_oid))
             .collect();
-        let post_rewrite_stdin = OsString::from(post_rewrite_stdin);
+        let post_rewrite_stdin = BString::from(post_rewrite_stdin);
         git_run_info.run_hook(
             effects,
             repo,
@@ -752,11 +843,9 @@ mod in_memory {
 }
 
 mod on_disk {
-    use std::ffi::OsStr;
     use std::fmt::Write;
 
     use eyre::Context;
-    use os_str_bytes::OsStrBytes;
     use tracing::instrument;
 
     use crate::core::effects::{Effects, OperationType};
@@ -845,9 +934,9 @@ mod on_disk {
                 &head_name_file_path,
                 head_info
                     .reference_name
-                    .as_deref()
-                    .unwrap_or_else(|| OsStr::new("detached HEAD"))
-                    .to_raw_bytes(),
+                    .as_ref()
+                    .map(|reference_name| reference_name.as_str())
+                    .unwrap_or("detached HEAD"),
             )
             .wrap_err_with(|| format!("Writing head-name to: {:?}", &head_name_file_path))?;
 
