@@ -20,12 +20,12 @@ use crate::revset::resolve_commits;
 use lib::core::config::{
     get_hint_enabled, get_restack_preserve_timestamps, print_hint_suppression_notice, Hint,
 };
-use lib::core::dag::{commit_set_to_vec_unsorted, sorted_commit_set, union_all, CommitSet, Dag};
+use lib::core::dag::{commit_set_to_vec, sorted_commit_set, union_all, CommitSet, Dag};
 use lib::core::effects::Effects;
 use lib::core::eventlog::{EventLogDb, EventReplayer};
 use lib::core::rewrite::{
     execute_rebase_plan, BuildRebasePlanOptions, ExecuteRebasePlanOptions, ExecuteRebasePlanResult,
-    MergeConflictRemediation, RebasePlanBuilder, RepoResource,
+    MergeConflictRemediation, RebasePlanBuilder, RebasePlanPermissions, RepoResource,
 };
 use lib::git::{GitRunInfo, NonZeroOid, Repo};
 
@@ -121,8 +121,7 @@ pub fn r#move(
 
             for component in dag.get_connected_components(&exact_oids)?.into_iter() {
                 let component_roots = dag.query().roots(component.clone())?;
-                let component_root = match commit_set_to_vec_unsorted(&component_roots)?.as_slice()
-                {
+                let component_root = match commit_set_to_vec(&component_roots)?.as_slice() {
                     [only_commit_oid] => *only_commit_oid,
                     _ => {
                         writeln!(
@@ -160,7 +159,7 @@ pub fn r#move(
     };
 
     let dest_oid: NonZeroOid = match resolve_commits(effects, &repo, &mut dag, vec![dest.clone()]) {
-        Ok(commit_sets) => match commit_set_to_vec_unsorted(&commit_sets[0])?.as_slice() {
+        Ok(commit_sets) => match commit_set_to_vec(&commit_sets[0])?.as_slice() {
             [only_commit_oid] => *only_commit_oid,
             other => {
                 let Revset(expr) = dest;
@@ -192,7 +191,7 @@ pub fn r#move(
     };
     let base_oids = {
         let mut result = Vec::new();
-        for base_oid in commit_set_to_vec_unsorted(&base_oids)? {
+        for base_oid in commit_set_to_vec(&base_oids)? {
             let merge_base_oid = dag.get_one_merge_base_oid(effects, &repo, base_oid, dest_oid)?;
             let base_commit_oid = resolve_base_commit(&dag, merge_base_oid, base_oid)?;
             result.push(CommitSet::from(base_commit_oid))
@@ -245,11 +244,37 @@ pub fn r#move(
     let pool = ThreadPoolBuilder::new().build()?;
     let repo_pool = RepoResource::new_pool(&repo)?;
     let rebase_plan = {
-        let mut builder = RebasePlanBuilder::new(&dag);
+        let build_options = BuildRebasePlanOptions {
+            force_rewrite_public_commits,
+            dump_rebase_constraints,
+            dump_rebase_plan,
+            detect_duplicate_commits_via_patch_id,
+        };
+        let permissions = {
+            let commits_to_move = source_oids.clone();
+            let commits_to_move = commits_to_move.union(&union_all(
+                &exact_components.values().cloned().collect::<Vec<_>>(),
+            ));
+            let commits_to_move = if insert {
+                commits_to_move.union(&dag.query().children(CommitSet::from(dest_oid))?)
+            } else {
+                commits_to_move
+            };
+
+            match RebasePlanPermissions::verify_rewrite_set(&dag, &build_options, &commits_to_move)?
+            {
+                Ok(permissions) => permissions,
+                Err(err) => {
+                    err.describe(effects, &repo)?;
+                    return Ok(ExitCode(1));
+                }
+            }
+        };
+        let mut builder = RebasePlanBuilder::new(&dag, permissions);
 
         let source_roots = dag.query().roots(source_oids.clone())?;
-        for source_root in commit_set_to_vec_unsorted(&source_roots)? {
-            builder.move_subtree(source_root, dest_oid)?;
+        for source_root in commit_set_to_vec(&source_roots)? {
+            builder.move_subtree(source_root, vec![dest_oid])?;
         }
 
         let component_roots: CommitSet = exact_components.keys().cloned().collect();
@@ -340,7 +365,7 @@ pub fn r#move(
                 .difference(component)
                 .difference(&dag.obsolete_commits);
 
-            for component_child in commit_set_to_vec_unsorted(&component_children)? {
+            for component_child in commit_set_to_vec(&component_children)? {
                 // If the range being extracted has any child commits, then we
                 // need to move each of those subtrees up to the parent commit
                 // of the range. If, however, we're inserting the range and the
@@ -354,11 +379,11 @@ pub fn r#move(
                 {
                     builder.move_range(component_child, component_dest_oid, component_parent)?;
                 } else {
-                    builder.move_subtree(component_child, component_parent)?;
+                    builder.move_subtree(component_child, vec![component_parent])?;
                 }
             }
 
-            builder.move_subtree(component_root, component_dest_oid)?;
+            builder.move_subtree(component_root, vec![component_dest_oid])?;
         }
 
         if insert {
@@ -395,7 +420,7 @@ pub fn r#move(
                     .query()
                     .heads(dag.query().descendants(source_oids.clone())?)?
                     .union(&exact_head);
-                match commit_set_to_vec_unsorted(&source_heads)?.as_slice() {
+                match commit_set_to_vec(&source_heads)?.as_slice() {
                     [oid] => *oid,
                     _ => {
                         writeln!(
@@ -420,21 +445,11 @@ pub fn r#move(
                 .difference(&exact_oids)
                 .difference(&dag.obsolete_commits);
 
-            for dest_child in commit_set_to_vec_unsorted(&dest_children)? {
-                builder.move_subtree(dest_child, source_head)?;
+            for dest_child in commit_set_to_vec(&dest_children)? {
+                builder.move_subtree(dest_child, vec![source_head])?;
             }
         }
-        builder.build(
-            effects,
-            &pool,
-            &repo_pool,
-            &BuildRebasePlanOptions {
-                force_rewrite_public_commits,
-                dump_rebase_constraints,
-                dump_rebase_plan,
-                detect_duplicate_commits_via_patch_id,
-            },
-        )?
+        builder.build(effects, &pool, &repo_pool)?
     };
     let result = match rebase_plan {
         Ok(None) => {
