@@ -3,7 +3,6 @@
 //! This is accomplished by finding the events that have happened since a certain
 //! time and inverting them.
 
-use std::ffi::OsString;
 use std::fmt::Write;
 use std::io::{stdin, BufRead, BufReader, Read};
 use std::sync::mpsc::{channel, Receiver, Sender, TryRecvError};
@@ -15,7 +14,7 @@ use cursive::utils::markup::StyledString;
 use cursive::views::{Dialog, EditView, LinearLayout, OnEventView, Panel, ScrollView, TextView};
 use cursive::{Cursive, CursiveRunnable, CursiveRunner};
 use eyre::Context;
-use lib::core::check_out::{check_out_commit, CheckOutCommitOptions};
+use lib::core::check_out::{check_out_commit, CheckOutCommitOptions, CheckoutTarget};
 use lib::core::repo_ext::RepoExt;
 use lib::util::ExitCode;
 use tracing::instrument;
@@ -150,7 +149,7 @@ fn describe_event(glyphs: &Glyphs, repo: &Repo, event: &Event) -> eyre::Result<V
             old_oid: MaybeZeroOid::Zero,
             new_oid: MaybeZeroOid::NonZero(new_oid),
             message: _,
-        } if ref_name == "HEAD" => {
+        } if ref_name.as_str() == "HEAD" => {
             // Not sure if this can happen. When a repo is created, maybe?
             vec![
                 StyledStringBuilder::new()
@@ -168,7 +167,7 @@ fn describe_event(glyphs: &Glyphs, repo: &Repo, event: &Event) -> eyre::Result<V
             old_oid: MaybeZeroOid::NonZero(old_oid),
             new_oid: MaybeZeroOid::NonZero(new_oid),
             message: _,
-        } if ref_name == "HEAD" => {
+        } if ref_name.as_str() == "HEAD" => {
             vec![
                 StyledStringBuilder::new()
                     .append_plain("Check out from ")
@@ -695,14 +694,14 @@ fn inverse_event(
 }
 
 #[derive(Clone, Debug)]
-struct CheckoutTarget {
-    target: OsString,
+struct UndoCheckoutTarget {
+    target: CheckoutTarget,
     options: CheckOutCommitOptions,
 }
 
 fn extract_checkout_target(
     events: &[Event],
-) -> eyre::Result<(Option<CheckoutTarget>, Vec<&Event>)> {
+) -> eyre::Result<(Option<UndoCheckoutTarget>, Vec<&Event>)> {
     let mut new_events = Vec::new();
     let mut checkout_target = None;
     for event in events.iter() {
@@ -712,11 +711,11 @@ fn extract_checkout_target(
                 event_tx_id: _,
                 ref_name,
                 old_oid: _,
-                new_oid,
+                new_oid: MaybeZeroOid::NonZero(new_oid),
                 message: _,
-            } if ref_name == "HEAD" => {
-                checkout_target = Some(CheckoutTarget {
-                    target: new_oid.to_string().into(),
+            } if ref_name.as_str() == "HEAD" => {
+                checkout_target = Some(UndoCheckoutTarget {
+                    target: CheckoutTarget::Oid(*new_oid),
                     options: CheckOutCommitOptions {
                         additional_args: vec!["--detach".into()],
                         render_smartlog: true,
@@ -731,13 +730,13 @@ fn extract_checkout_target(
                 commit_oid,
                 ref_name,
             } => {
-                checkout_target = Some(CheckoutTarget {
-                    target: commit_oid.to_string().into(),
+                checkout_target = Some(UndoCheckoutTarget {
+                    target: CheckoutTarget::Oid(*commit_oid),
                     options: CheckOutCommitOptions {
                         additional_args: match ref_name {
                             Some(ref_name) => {
                                 let branch_name = CategorizedReferenceName::new(ref_name);
-                                vec!["-B".into(), branch_name.remove_prefix()?]
+                                vec!["-B".into(), branch_name.remove_prefix()?.into()]
                             }
                             None => Default::default(),
                         },
@@ -781,7 +780,7 @@ fn undo_events(
                     old_oid: MaybeZeroOid::Zero,
                     new_oid: _,
                     message: _,
-                } if ref_name == "HEAD"
+                } if ref_name.as_str() == "HEAD"
             )
         })
         .map(|event| inverse_event(event.clone(), now, event_tx_id))
@@ -861,7 +860,7 @@ fn undo_events(
                     writeln!(
                         effects.get_output_stream(),
                         "Reference {} did not exist, not deleting it.",
-                        ref_name.to_string_lossy()
+                        ref_name.as_str()
                     )?;
                 }
             },
@@ -898,7 +897,7 @@ fn undo_events(
         }
     }
 
-    if let Some(CheckoutTarget { target, options }) = checkout_target {
+    if let Some(UndoCheckoutTarget { target, options }) = checkout_target {
         let exit_code = check_out_commit(
             effects,
             git_run_info,
@@ -1050,8 +1049,10 @@ mod tests {
         insta::assert_debug_snapshot!(extract_checkout_target(&input)?, @r###"
         (
             Some(
-                CheckoutTarget {
-                    target: "3000000000000000000000000000000000000000",
+                UndoCheckoutTarget {
+                    target: Oid(
+                        NonZeroOid(3000000000000000000000000000000000000000),
+                    ),
                     options: CheckOutCommitOptions {
                         additional_args: [
                             "--detach",
