@@ -17,9 +17,10 @@
 //! Then the branchless workflow may be for you instead.
 
 #![warn(missing_docs)]
-#![warn(clippy::all, clippy::as_conversions)]
+#![warn(clippy::all, clippy::as_conversions, clippy::clone_on_ref_ptr)]
 #![allow(clippy::too_many_arguments, clippy::blocks_in_if_conditions)]
 
 pub mod commands;
 pub mod opts;
+pub mod revset;
 pub mod tui;
