@@ -3,6 +3,7 @@
 use clap::{ArgEnum, Args, Command as ClapCommand, IntoApp, Parser};
 use lib::git::NonZeroOid;
 use man::Arg;
+use std::fmt::Display;
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
 
@@ -19,6 +20,12 @@ impl FromStr for Revset {
     }
 }
 
+impl Display for Revset {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}", self.0)
+    }
+}
+
 /// A command wrapped by `git-branchless wrap`. The arguments are forwarded to
 /// `git`.
 #[derive(Parser)]
@@ -336,7 +343,14 @@ pub enum Command {
         traverse_commits_options: TraverseCommitsOptions,
     },
 
-    /// Query the commit graph using the "revset" DSL.
+    /// Query the commit graph using the "revset" language and print matching
+    /// commits.
+    ///
+    /// See https://github.com/arxanas/git-branchless/wiki/Reference:-Revsets to
+    /// learn more about revsets.
+    ///
+    /// The outputted commits are guaranteed to be topologically sorted, with
+    /// ancestor commits appearing first.
     Query {
         /// The query to execute.
         #[clap(value_parser)]
@@ -383,13 +397,15 @@ pub enum Command {
         /// current staged/unstaged changes.
         #[clap(action, short = 'i', long = "interactive")]
         interactive: bool,
+
+        /// Detach the current branch before committing.
+        #[clap(action, short = 'd', long = "detach")]
+        detach: bool,
     },
 
     /// Reword commits.
     Reword {
         /// Zero or more commits to reword. If not provided, defaults to "HEAD".
-        ///
-        /// Can either be hashes, like `abc123`, or ref-specs, like `HEAD^`.
         #[clap(value_parser)]
         revsets: Vec<Revset>,
 
@@ -409,6 +425,11 @@ pub enum Command {
         /// that; otherwise, the editor starts empty.
         #[clap(action, short = 'd', long = "discard", conflicts_with("messages"))]
         discard: bool,
+
+        /// A commit to "fix up". The reworded commits will become `fixup!` commits (suitable for
+        /// use with `git rebase --autosquash`) targeting the supplied commit.
+        #[clap(value_parser, long = "fixup", conflicts_with_all(&["messages", "discard"]))]
+        commit_to_fixup: Option<Revset>,
     },
 
     /// Display a nice graph of the commits you've recently worked on.
@@ -437,6 +458,23 @@ pub enum Command {
         subcommand: SnapshotSubcommand,
     },
 
+    /// Push commits to a remote.
+    Submit {
+        /// If there is no remote branch for a given local branch, create the
+        /// remote branch by pushing the local branch to the default push
+        /// remote.
+        ///
+        /// You can configure the default push remote with `git config
+        /// remote.pushDefault <remote>`.
+        #[clap(action, short = 'c', long = "create")]
+        create: bool,
+
+        /// The commits to push. All branches attached to those commits will be
+        /// pushed.
+        #[clap(value_parser, default_value = "stack()")]
+        revset: Revset,
+    },
+
     /// Move any local commit stacks on top of the main branch.
     Sync {
         /// Run `git fetch` to update remote references before carrying out the
@@ -515,11 +553,11 @@ pub enum ColorSetting {
 pub struct Opts {
     /// Change to the given directory before executing the rest of the program.
     /// (The option is called `-C` for symmetry with Git.)
-    #[clap(value_parser, short = 'C')]
+    #[clap(value_parser, short = 'C', global = true)]
     pub working_directory: Option<PathBuf>,
 
     /// Flag to force enable or disable terminal colors.
-    #[clap(value_parser, long = "color", arg_enum)]
+    #[clap(value_parser, long = "color", arg_enum, global = true)]
     pub color: Option<ColorSetting>,
 
     /// The `git-branchless` subcommand to run.
